@@ -148,12 +148,13 @@ comment_data_dup (CommentData *data)
 
 char *
 comments_get_comment_filename (const char *source,
-			       gboolean    resolve_symlinks) 
+			       gboolean    resolve_symlinks,
+			       gboolean    unescape) 
 {
 	char        *source_real = NULL;
-	char        *directory;
-	const char  *filename;
-	char        *path;
+	char        *directory = NULL;
+	const char  *filename = NULL;
+	char        *path = NULL;
 
 	if (source == NULL) 
 		return NULL;
@@ -162,18 +163,13 @@ comments_get_comment_filename (const char *source,
 
 	if (resolve_symlinks) {
 		char           *parent;
-		char           *parent_escaped;
 		char           *parent_resolved;
 		GnomeVFSResult  result;
 
 		parent = remove_level_from_path (source);
-		parent_escaped = gnome_vfs_escape_path_string (parent);
+		result = resolve_all_symlinks (parent, &parent_resolved);
 		g_free (parent);
 
-		result = resolve_all_symlinks (parent_escaped, 
-					       &parent_resolved);
-		g_free (parent_escaped);
-
 		if (result == GNOME_VFS_OK) { 
 			g_free (source_real);
 			source_real = g_strconcat (parent_resolved, 
@@ -199,13 +195,20 @@ comments_get_comment_filename (const char *source,
 	g_free (directory);
 	g_free (source_real);
 
+	if (unescape) {
+		char *unesc_path = gnome_vfs_unescape_string (path, NULL);
+		g_free (path);
+		path = unesc_path;
+	}
+
 	return path;
 }
 
 
 char *
 comments_get_comment_dir (const char *directory,
-			  gboolean    resolve_symlinks) 
+			  gboolean    resolve_symlinks,
+			  gboolean    unescape) 
 {
 	char        *directory_resolved = NULL;
 	const char  *directory_real = directory;
@@ -213,16 +216,13 @@ comments_get_comment_dir (const char *directory,
 	char        *path;
 
 	if (resolve_symlinks && (directory != NULL)) {
-		char           *directory_escaped;
 		GnomeVFSResult  result;
-
-		directory_escaped = gnome_vfs_escape_path_string (directory);
-		result = resolve_all_symlinks (directory_escaped, 
-					       &directory_resolved);
-		g_free (directory_escaped);
-
+		result = resolve_all_symlinks (directory, &directory_resolved);
 		if (result == GNOME_VFS_OK) 
 			directory_real = directory_resolved;
+		else
+			g_print ("ERROR: %s\n", gnome_vfs_result_to_string (result));
+		g_print ("RESOLVED: %s\n", directory_resolved);
 	}
 
 	if (directory_real == NULL)
@@ -239,6 +239,12 @@ comments_get_comment_dir (const char *directory,
 
 	g_free (directory_resolved);
 
+	if (unescape) {
+		char *unesc_path = gnome_vfs_unescape_string (path, NULL);
+		g_free (path);
+		path = unesc_path;
+	}
+
 	return path;
 }
 
@@ -247,19 +253,19 @@ void
 comment_copy (const char *src,
 	      const char *dest)
 {
-	char *comment_src;
-	char *comment_dest;
+	char *comment_src = NULL;
+	char *comment_dest = NULL;
 
-	comment_src = comments_get_comment_filename (src, TRUE);
+	comment_src = comments_get_comment_filename (src, TRUE, TRUE);
 	if (! path_is_file (comment_src)) {
 		g_free (comment_src);
-		comment_src = comments_get_comment_filename (src, FALSE);
+		comment_src = comments_get_comment_filename (src, FALSE, TRUE);
 		if (! path_is_file (comment_src)) {
 			g_free (comment_src);
 			return;
 		}
 	}
-	comment_dest = comments_get_comment_filename (dest, TRUE);
+	comment_dest = comments_get_comment_filename (dest, TRUE, TRUE);
 
 	if (path_is_file (comment_dest)) 
 		unlink (comment_dest);
@@ -274,20 +280,20 @@ void
 comment_move (const char *src,
 	      const char *dest)
 {
-	char *comment_src;
-	char *comment_dest;
+	char *comment_src = NULL;
+	char *comment_dest = NULL;
 
-	comment_src = comments_get_comment_filename (src, TRUE);
+	comment_src = comments_get_comment_filename (src, TRUE, TRUE);
 	if (! path_is_file (comment_src)) {
 		g_free (comment_src);
-		comment_src = comments_get_comment_filename (src, FALSE);
+		comment_src = comments_get_comment_filename (src, FALSE, TRUE);
 		if (! path_is_file (comment_src)) {
 			g_free (comment_src);
 			return;
 		}
 	}
 
-	comment_dest = comments_get_comment_filename (dest, TRUE);
+	comment_dest = comments_get_comment_filename (dest, TRUE, TRUE);
 
 	if (path_is_file (comment_dest)) 
 		unlink (comment_dest);
@@ -303,11 +309,11 @@ comment_delete (const char *filename)
 {
 	char *comment_name;
 
-	comment_name = comments_get_comment_filename (filename, FALSE);
+	comment_name = comments_get_comment_filename (filename, FALSE, TRUE);
 	unlink (comment_name);
 	g_free (comment_name);
 
-	comment_name = comments_get_comment_filename (filename, TRUE);
+	comment_name = comments_get_comment_filename (filename, TRUE, TRUE);
 	unlink (comment_name);
 	g_free (comment_name);
 }
@@ -336,6 +342,7 @@ comments_remove_old_comments (const char *dir,
 typedef struct {
 	gboolean   recursive;
 	gboolean   clear_all;
+	gboolean   interrupted;
 	GtkWidget *dialog;
 } CommentsRemoveData;
 
@@ -354,17 +361,16 @@ check_comment_file (char     *real_file,
 }
 
 
-void
+static void
 remove_comments_done (const GList *dir_list, 
 		      gpointer     data)
 {
 	CommentsRemoveData *crd = data;
 	const GList        *scan;
 	
-	if (! crd->clear_all) {
+	if (! crd->clear_all || crd->interrupted) {
 		gtk_widget_destroy (crd->dialog);
 		g_free (crd);
-
 		return;
 	}
 
@@ -375,6 +381,15 @@ remove_comments_done (const GList *dir_list,
 }
 
 
+static void
+crd_interrupt_cb (GtkDialog *dialog,
+		  int response_id,
+		  CommentsRemoveData *crd)
+{
+	crd->interrupted = TRUE;
+}
+
+
 void
 comments_remove_old_comments_async (const char *dir,
 				    gboolean    recursive,
@@ -395,12 +410,13 @@ comments_remove_old_comments_async (const char *dir,
 					       GTK_DIALOG_MODAL,
 					       GTK_MESSAGE_INFO,
 					       message,
+					       NULL,
 					       GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 					       NULL);
-	g_signal_connect_swapped (G_OBJECT (crd->dialog),
-				  "response",
-				  G_CALLBACK (gtk_widget_hide),
-				  crd->dialog);
+	g_signal_connect (G_OBJECT (crd->dialog),
+			  "response",
+			  G_CALLBACK (crd_interrupt_cb),
+			  crd);
 	gtk_widget_show (crd->dialog);
 
 	visit_rc_directory_async (RC_COMMENTS_DIR,
@@ -493,11 +509,11 @@ comments_load_comment (const char *filename)
 	if (filename == NULL)
 		return NULL;
 
-	comment_file = comments_get_comment_filename (filename, TRUE);
+	comment_file = comments_get_comment_filename (filename, TRUE, TRUE);
 	if (! path_is_file (comment_file)) {
 		g_free (comment_file);
 
-		comment_file = comments_get_comment_filename (filename, FALSE);
+		comment_file = comments_get_comment_filename (filename, FALSE, TRUE);
 		if (! path_is_file (comment_file)) {
 			g_free (comment_file);
 
@@ -559,10 +575,11 @@ save_comment (const char  *filename,
 {
 	xmlDocPtr    doc;
         xmlNodePtr   tree, subtree;
-	char        *comment_file;
-	char        *time_str;
-	char        *keywords_str;
-	char        *dest_dir;
+	char        *comment_file = NULL;
+	char        *time_str = NULL;
+	char        *keywords_str = NULL;
+	char        *dest_dir = NULL;
+	char        *e_comment = NULL, *e_place = NULL, *e_keywords = NULL;
 
 	if (comment_data_is_void (data)) {
 		comment_delete (filename);
@@ -580,6 +597,18 @@ save_comment (const char  *filename,
 	} else
 		keywords_str = g_strdup ("");
 
+	/* Escape text */
+
+	if (data->comment != NULL)
+		e_comment = g_markup_escape_text (data->comment, -1);
+
+	if (data->place != NULL)
+		e_place = g_markup_escape_text (data->place, -1);
+
+	if (keywords_str != NULL)
+		e_keywords = g_markup_escape_text (keywords_str, -1);
+	g_free (keywords_str);
+
 	/* Create the xml tree. */
 
 	doc = xmlNewDoc ("1.0");
@@ -588,17 +617,19 @@ save_comment (const char  *filename,
 	xmlSetProp (doc->xmlRootNode, FORMAT_TAG, FORMAT_VER);
 
 	tree = doc->xmlRootNode;
-	subtree = xmlNewChild (tree, NULL, PLACE_TAG,    data->place);
+	subtree = xmlNewChild (tree, NULL, PLACE_TAG,    e_place);
 	subtree = xmlNewChild (tree, NULL, TIME_TAG,     time_str);
-	subtree = xmlNewChild (tree, NULL, NOTE_TAG,     data->comment);
-	subtree = xmlNewChild (tree, NULL, KEYWORDS_TAG, keywords_str);
+	subtree = xmlNewChild (tree, NULL, NOTE_TAG,     e_comment);
+	subtree = xmlNewChild (tree, NULL, KEYWORDS_TAG, e_keywords);
 
+	g_free (e_place);
 	g_free (time_str);
-	g_free (keywords_str);
+	g_free (e_comment);
+	g_free (e_keywords);
 
 	/* Write to disk. */
 
-	comment_file = comments_get_comment_filename (filename, TRUE);
+	comment_file = comments_get_comment_filename (filename, TRUE, TRUE);
 	dest_dir = remove_level_from_path (comment_file);
 	if (ensure_dir_exists (dest_dir, 0700)) {
 		xmlSetDocCompressMode (doc, 3);
