@@ -41,7 +41,8 @@
 #include "gthumb-window.h"
 #include "gth-folder-selection-dialog.h"
 
-#define GLADE_EXPORTER_FILE    "gthumb_png_exporter.glade"
+#define GLADE_FILE "gthumb.glade"
+#define GLADE_PNG_EXP_FILE "gthumb_png_exporter.glade"
 #define DISPLAY_PROGRESS_DELAY 1000
 #define FILE_NAME_MAX_LENGTH   30
 #define PREVIEW_SIZE           150
@@ -98,7 +99,7 @@ dlg_check_folder (GThumbWindow *window,
 
 	if (! ensure_dir_exists (dir, 0755)) {
 		char *utf8_path;
-		utf8_path = g_locale_to_utf8 (dir, -1, NULL, NULL, NULL);
+		utf8_path = g_filename_to_utf8 (dir, -1, NULL, NULL, NULL);
 		_gtk_error_dialog_run (GTK_WINDOW (window->app),
 				       _("Could not create folder \"%s\": %s."),
 				       utf8_path,
@@ -112,7 +113,7 @@ dlg_check_folder (GThumbWindow *window,
 
 	if (access (dir, R_OK | W_OK | X_OK) != 0) {
 		char *utf8_path;
-		utf8_path = g_locale_to_utf8 (dir, -1, NULL, NULL, NULL);
+		utf8_path = g_filename_to_utf8 (dir, -1, NULL, NULL, NULL);
 		_gtk_error_dialog_run (GTK_WINDOW (window->app),
 				       _("You don't have the right permissions to create images in the folder \"%s\""),
 				       utf8_path);
@@ -168,6 +169,7 @@ dlg_show_error (GThumbWindow *window,
 				     GTK_DIALOG_MODAL,
 				     GTK_STOCK_DIALOG_ERROR,
 				     utf8_msg,
+				     NULL,
 				     GTK_STOCK_OK, GTK_RESPONSE_CANCEL,
 				     NULL);
 	g_free (msg);
@@ -211,17 +213,23 @@ real_files_delete__continue (GnomeVFSResult result,
 	ConfirmFileDeleteData *cfddata = data;
 
 	if (result == GNOME_VFS_ERROR_NOT_FOUND) {
-		GtkWidget *d;
-		int        r;
+		int r = GTK_RESPONSE_YES;
 
-		d = _gtk_yesno_dialog_new (GTK_WINDOW (cfddata->window->app),
-					   GTK_DIALOG_MODAL,
-					   _("The images cannot be moved to the Trash. Do you want to delete them permanently?"),
-					   GTK_STOCK_CANCEL,
-					   GTK_STOCK_DELETE);
+		if (eel_gconf_get_boolean (PREF_MSG_CANNOT_MOVE_TO_TRASH, TRUE)) {
+			GtkWidget *d;
 
-		r = gtk_dialog_run (GTK_DIALOG (d));
-		gtk_widget_destroy (GTK_WIDGET (d));
+			d = _gtk_yesno_dialog_with_checkbutton_new (
+				    GTK_WINDOW (cfddata->window->app),
+				    GTK_DIALOG_MODAL,
+				    _("The images cannot be moved to the Trash. Do you want to delete them permanently?"),
+				    GTK_STOCK_CANCEL,
+				    GTK_STOCK_DELETE,
+				    _("_Do not display this message again"),
+				    PREF_MSG_CANNOT_MOVE_TO_TRASH);
+		
+			r = gtk_dialog_run (GTK_DIALOG (d));
+			gtk_widget_destroy (GTK_WIDGET (d));
+		} 
 
 		if (r == GTK_RESPONSE_YES) 
 			dlg_files_delete (cfddata->window, 
@@ -261,7 +269,7 @@ dlg_file_delete__confirm (GThumbWindow *window,
 	GtkWidget *dialog;
 	int        result;
 
-	if (! eel_gconf_get_boolean (PREF_CONFIRM_DELETION)) {
+	if (! eel_gconf_get_boolean (PREF_CONFIRM_DELETION, TRUE)) {
 		real_files_delete (window, list);
 		return TRUE;
 	}
@@ -498,10 +506,10 @@ set_filename_labels (GladeXML    *gui,
 	eventbox = glade_xml_get_widget (gui, filename_eventbox);
 
 	name = _g_strdup_with_max_size (file_name_from_path (filename), FILE_NAME_MAX_LENGTH);
-	_gtk_label_set_locale_text (GTK_LABEL (label), name);
+	_gtk_label_set_filename_text (GTK_LABEL (label), name);
 	g_free (name);
 
-	utf8_name = g_locale_to_utf8 (filename, -1, NULL, NULL, NULL);
+	utf8_name = g_filename_to_utf8 (filename, -1, NULL, NULL, NULL);
 	gtk_tooltips_set_tip (tooltips, eventbox, utf8_name, NULL);
 	g_free (utf8_name);
 
@@ -666,8 +674,8 @@ create_overwrite_dialog (GThumbWindow      *window,
 
 	gtk_widget_set_sensitive (owdata->overwrite_rename_entry,
 				  default_overwrite_mode == OVERWRITE_RESULT_RENAME);
-	_gtk_entry_set_locale_text (GTK_ENTRY (owdata->overwrite_rename_entry),
-				    file_name_from_path (new_filename));
+	_gtk_entry_set_filename_text (GTK_ENTRY (owdata->overwrite_rename_entry),
+				      file_name_from_path (new_filename));
 	if (default_overwrite_mode == OVERWRITE_RESULT_RENAME)
 		gtk_widget_grab_focus (owdata->overwrite_rename_entry);
 
@@ -744,7 +752,7 @@ dlg_overwrite__response_cb (GtkWidget *dialog,
 			char *new_path;
 
 			result = OVERWRITE_RESULT_RENAME;
-			new_name = _gtk_entry_get_locale_text (GTK_ENTRY (owdata->overwrite_rename_entry));
+			new_name = _gtk_entry_get_filename_text (GTK_ENTRY (owdata->overwrite_rename_entry));
 			if ((new_name == NULL) || (new_name[0] == 0)) {
 				_gtk_error_dialog_run (GTK_WINDOW (owdata->window->app),
 						       _("You didn't enter the new name"));
@@ -757,7 +765,7 @@ dlg_overwrite__response_cb (GtkWidget *dialog,
 			if (path_is_file (new_path)) {
 				char *utf8_name;
 
-				utf8_name = g_locale_to_utf8 (new_name, -1, 0, 0, 0);
+				utf8_name = g_filename_to_utf8 (new_name, -1, 0, 0, 0);
 				_gtk_error_dialog_run (GTK_WINDOW (owdata->window->app),
 						       _("The name \"%s\" is already used in this folder. Please use a different name."),
 						       utf8_name);
@@ -844,7 +852,7 @@ dlg_overwrite_run (GThumbWindow  *window,
 		char *new_path;
 
 		result = OVERWRITE_RESULT_RENAME;
-		*new_name = _gtk_entry_get_locale_text (GTK_ENTRY (owdata->overwrite_rename_entry));
+		*new_name = _gtk_entry_get_filename_text (GTK_ENTRY (owdata->overwrite_rename_entry));
 
 		if ((*new_name == NULL) || ((*new_name)[0] == 0)) {
 			_gtk_error_dialog_run (GTK_WINDOW (owdata->window->app),
@@ -858,7 +866,7 @@ dlg_overwrite_run (GThumbWindow  *window,
 		if (path_is_file (new_path)) {
 			char *utf8_name;
 			
-			utf8_name = g_locale_to_utf8 (*new_name, -1, 0, 0, 0);
+			utf8_name = g_filename_to_utf8 (*new_name, -1, 0, 0, 0);
 			_gtk_error_dialog_run (GTK_WINDOW (owdata->window->app),
 					       _("The name \"%s\" is already used in this folder. Please use a different name."),
 					       utf8_name);
@@ -1037,6 +1045,7 @@ typedef struct {
 	GtkWidget           *progress_dialog;
 	GtkWidget           *progress_progressbar;
 	GtkWidget           *progress_info;
+	GtkWidget           *progress_cancel;
 
 	guint                timeout_id;
 
@@ -1137,7 +1146,7 @@ continue_or_abort_dialog (FileCopyData   *fcdata,
 		error = _("Could not move the image:");
 	else
 		error = _("Could not copy the image:");
-	utf8_name = g_locale_to_utf8 (file_name_from_path (src_file), -1, 0, 0, 0);
+	utf8_name = g_filename_to_utf8 (file_name_from_path (src_file), -1, 0, 0, 0);
 	message = g_strconcat (error, 
 			       " ",
 			       utf8_name, 
@@ -1153,6 +1162,7 @@ continue_or_abort_dialog (FileCopyData   *fcdata,
 					     GTK_DIALOG_MODAL,
 					     GTK_STOCK_DIALOG_ERROR,
 					     message,
+					     NULL,
 					     GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL,
 					     NULL);
 	else 
@@ -1182,9 +1192,11 @@ files_copy__done (FileCopyData *fcdata)
 	if (fcdata->remove_source)
 		all_windows_notify_files_deleted (fcdata->copied_list);
 	all_windows_notify_files_created (fcdata->created_list);
-	
+	all_windows_add_monitor ();
+
 	if (fcdata->done_func != NULL)
 		(*fcdata->done_func) (fcdata->result, fcdata->done_data);
+	
 	file_copy_data_free (fcdata);
 }
 
@@ -1339,7 +1351,8 @@ copy_current_file (FileCopyData *fcdata)
 	g_free (message);
 
 	fraction = (float) fcdata->file_index / fcdata->files_total;
-	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (fcdata->progress_progressbar), fraction);
+	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (fcdata->progress_progressbar), 
+				       CLAMP (fraction, 0.0, 1.0));
 
 	/**/
 
@@ -1437,13 +1450,13 @@ copy_next_file (FileCopyData *fcdata)
 					 file_name_from_path (src_file), 
 					 NULL);
 		
-		src_cache_file = comments_get_comment_filename (src_file, TRUE);
+		src_cache_file = comments_get_comment_filename (src_file, TRUE, TRUE);
 		if (! path_is_file (src_cache_file)) {
 			g_free (src_cache_file);
-			src_cache_file = comments_get_comment_filename (src_file, FALSE);
+			src_cache_file = comments_get_comment_filename (src_file, FALSE, TRUE);
 		}
 
-		dest_cache_file = comments_get_comment_filename (dest_file, TRUE);
+		dest_cache_file = comments_get_comment_filename (dest_file, TRUE, TRUE);
 
 		src_cache_dir = remove_level_from_path (src_cache_file);
 
@@ -1518,6 +1531,32 @@ copy_next_file (FileCopyData *fcdata)
 }
 
 
+static void
+dlg_files_copy__interrupt (FileCopyData *fcdata)
+{
+	if (fcdata->handle != NULL)
+		gnome_vfs_async_cancel (fcdata->handle);
+	files_copy__done (fcdata);
+}
+
+
+static void
+dlg_files_copy__interrupt_cb1 (GtkWidget    *caller, 
+			       GdkEvent     *event, 
+			       FileCopyData *fcdata)
+{
+	dlg_files_copy__interrupt (fcdata);
+}
+
+
+static void
+dlg_files_copy__interrupt_cb2 (GtkWidget    *caller, 
+			       FileCopyData *fcdata)
+{
+	dlg_files_copy__interrupt (fcdata);
+}
+
+
 void
 dlg_files_copy (GThumbWindow   *window,
 		GList          *file_list,
@@ -1528,7 +1567,7 @@ dlg_files_copy (GThumbWindow   *window,
 		FileOpDoneFunc  done_func,
 		gpointer        done_data)
 {
-	FileCopyData              *fcdata;
+	FileCopyData *fcdata;
 
 	if (! path_is_dir (dest_path))
 		return;
@@ -1565,16 +1604,26 @@ dlg_files_copy (GThumbWindow   *window,
 
 	/**/
 
-	fcdata->gui = glade_xml_new (GTHUMB_GLADEDIR "/" GLADE_EXPORTER_FILE, NULL, NULL);
+	fcdata->gui = glade_xml_new (GTHUMB_GLADEDIR "/" GLADE_PNG_EXP_FILE, NULL, NULL);
 	if (! fcdata->gui) {
 		file_copy_data_free (fcdata);
-		g_warning ("Could not find " GLADE_EXPORTER_FILE "\n");
+		g_warning ("Could not find " GLADE_PNG_EXP_FILE "\n");
 		return;
 	}
 
 	fcdata->progress_dialog = glade_xml_get_widget (fcdata->gui, "progress_dialog");
 	fcdata->progress_progressbar = glade_xml_get_widget (fcdata->gui, "progress_progressbar");
 	fcdata->progress_info = glade_xml_get_widget (fcdata->gui, "progress_info");
+	fcdata->progress_cancel = glade_xml_get_widget (fcdata->gui, "progress_cancel");
+
+	g_signal_connect (G_OBJECT (fcdata->progress_dialog), 
+			  "delete_event",
+			  G_CALLBACK (dlg_files_copy__interrupt_cb1),
+			  fcdata);
+	g_signal_connect (G_OBJECT (fcdata->progress_cancel), 
+			  "clicked",
+			  G_CALLBACK (dlg_files_copy__interrupt_cb2),
+			  fcdata);
 
 	gtk_window_set_transient_for (GTK_WINDOW (fcdata->progress_dialog),
 				      GTK_WINDOW (fcdata->window->app));
@@ -1584,6 +1633,8 @@ dlg_files_copy (GThumbWindow   *window,
 					    file_copy__display_progress_dialog,
 					    fcdata);
 
+	all_windows_remove_monitor ();
+
 	copy_current_file (fcdata);
 }
 
@@ -1642,6 +1693,7 @@ typedef struct {
 	GtkWidget           *progress_dialog;
 	GtkWidget           *progress_progressbar;
 	GtkWidget           *progress_info;
+	GtkWidget           *progress_cancel;
 
 	GThumbWindow        *window;
 	GList               *file_list;
@@ -1692,11 +1744,16 @@ file_delete__display_progress_dialog (gpointer data)
 static void
 files_delete__done (FileDeleteData *fddata)
 {
-	if (fddata->result == GNOME_VFS_OK)
+	if (fddata->result == GNOME_VFS_OK) 
 		all_windows_notify_files_deleted (fddata->file_list);
-	
+	else
+		all_windows_notify_files_changed (fddata->file_list);
+
+	/*all_windows_add_monitor (); FIXME*/
+
 	if (fddata->done_func != NULL)
 		(*fddata->done_func) (fddata->result, fddata->done_data);
+
 	file_delete_data_free (fddata);
 }
 
@@ -1734,13 +1791,40 @@ file_delete_progress_update_cb (GnomeVFSAsyncHandle      *handle,
 
 	if (info->bytes_total != 0) {
 		fraction = (float)info->total_bytes_copied / info->bytes_total;
-		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (fddata->progress_progressbar), fraction);
+		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (fddata->progress_progressbar), 
+					       CLAMP (fraction, 0.0, 1.0));
 	}
 
 	return TRUE;
 }
 
 
+static void
+dlg_files_delete__interrupt (FileDeleteData *fcdata)
+{
+	if (fcdata->handle != NULL)
+		gnome_vfs_async_cancel (fcdata->handle);
+	files_delete__done (fcdata);
+}
+
+
+static void
+dlg_files_delete__interrupt_cb1 (GtkWidget      *caller, 
+				 GdkEvent       *event, 
+				 FileDeleteData *fcdata)
+{
+	dlg_files_delete__interrupt (fcdata);
+}
+
+
+static void
+dlg_files_delete__interrupt_cb2 (GtkWidget      *caller, 
+				 FileDeleteData *fcdata)
+{
+	dlg_files_delete__interrupt (fcdata);
+}
+
+
 void
 dlg_files_delete (GThumbWindow   *window,
 		  GList          *file_list,
@@ -1768,16 +1852,26 @@ dlg_files_delete (GThumbWindow   *window,
 
 	/**/
 
-	fddata->gui = glade_xml_new (GTHUMB_GLADEDIR "/" GLADE_EXPORTER_FILE, NULL, NULL);
+	fddata->gui = glade_xml_new (GTHUMB_GLADEDIR "/" GLADE_PNG_EXP_FILE, NULL, NULL);
 	if (! fddata->gui) {
 		file_delete_data_free (fddata);
-		g_warning ("Could not find " GLADE_EXPORTER_FILE "\n");
+		g_warning ("Could not find " GLADE_PNG_EXP_FILE "\n");
 		return;
 	}
 
 	fddata->progress_dialog = glade_xml_get_widget (fddata->gui, "progress_dialog");
 	fddata->progress_progressbar = glade_xml_get_widget (fddata->gui, "progress_progressbar");
 	fddata->progress_info = glade_xml_get_widget (fddata->gui, "progress_info");
+	fddata->progress_cancel = glade_xml_get_widget (fddata->gui, "progress_cancel");
+
+	g_signal_connect (G_OBJECT (fddata->progress_dialog), 
+			  "delete_event",
+			  G_CALLBACK (dlg_files_delete__interrupt_cb1),
+			  fddata);
+	g_signal_connect (G_OBJECT (fddata->progress_cancel), 
+			  "clicked",
+			  G_CALLBACK (dlg_files_delete__interrupt_cb2),
+			  fddata);
 	
 	gtk_window_set_transient_for (GTK_WINDOW (fddata->progress_dialog),
 				      GTK_WINDOW (fddata->window->app));
@@ -1789,6 +1883,8 @@ dlg_files_delete (GThumbWindow   *window,
 
 	/**/
 
+	/*all_windows_remove_monitor (); FIXME*/
+
 	for (scan = fddata->file_list; scan; scan = scan->next) {
 		const char  *path = scan->data;
 		GnomeVFSURI *uri;
@@ -1848,6 +1944,7 @@ typedef struct {
 	GtkWidget           *progress_dialog;
 	GtkWidget           *progress_progressbar;
 	GtkWidget           *progress_info;
+	GtkWidget           *progress_cancel;
 
 	GnomeVFSAsyncHandle *handle;
 	GnomeVFSResult       result;
@@ -1872,6 +1969,8 @@ destroy_progress_dialog (FolderCopyData *fcdata)
 static void
 folder_copy_data_free (FolderCopyData *fcdata)
 {
+	all_windows_add_monitor ();
+
 	if (fcdata == NULL)
 		return;
 
@@ -1931,6 +2030,7 @@ folder_progress_update_cb (GnomeVFSAsyncHandle      *handle,
 		}
 
 		destroy_progress_dialog (fcdata);		
+
 		if (fcdata->done_func != NULL)
 			(*fcdata->done_func) (fcdata->result, fcdata->done_data);
 		folder_copy_data_free (fcdata);
@@ -1943,10 +2043,10 @@ folder_progress_update_cb (GnomeVFSAsyncHandle      *handle,
 		g_free (message);
 	}
 
-	if (info->bytes_total != 0) {
+	if (info->files_total != 0) {
 		fraction = (float) info->file_index / info->files_total;
-		fraction /= 2.0;
-		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (fcdata->progress_progressbar), fraction);
+		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (fcdata->progress_progressbar), 
+					       CLAMP (fraction, 0.0, 1.0));
 	}
 
 	return result;
@@ -1989,6 +2089,32 @@ folder_copy__display_progress_dialog (gpointer data)
 }
 
 
+static void
+folder_copy__interrupt (FolderCopyData *fcdata)
+{
+	if (fcdata->handle != NULL)
+		gnome_vfs_async_cancel (fcdata->handle);
+	folder_copy_data_free (fcdata);
+}
+
+
+static void
+folder_copy__interrupt_cb1 (GtkWidget     *caller, 
+			    GdkEvent       *event, 
+			    FolderCopyData *fcdata)
+{
+	folder_copy__interrupt (fcdata);
+}
+
+
+static void
+folder_copy__interrupt_cb2 (GtkWidget      *caller, 
+			    FolderCopyData *fcdata)
+{
+	folder_copy__interrupt (fcdata);
+}
+
+
 static void
 folder_copy (GThumbWindow   *window,
 	     const char     *src_path,
@@ -2031,16 +2157,26 @@ folder_copy (GThumbWindow   *window,
 
 	/**/
 
-	fcdata->gui = glade_xml_new (GTHUMB_GLADEDIR "/" GLADE_EXPORTER_FILE, NULL, NULL);
+	fcdata->gui = glade_xml_new (GTHUMB_GLADEDIR "/" GLADE_PNG_EXP_FILE, NULL, NULL);
 	if (! fcdata->gui) {
 		folder_copy_data_free (fcdata);
-		g_warning ("Could not find " GLADE_EXPORTER_FILE "\n");
+		g_warning ("Could not find " GLADE_PNG_EXP_FILE "\n");
 		return;
 	}
 
 	fcdata->progress_dialog = glade_xml_get_widget (fcdata->gui, "progress_dialog");
 	fcdata->progress_progressbar = glade_xml_get_widget (fcdata->gui, "progress_progressbar");
 	fcdata->progress_info = glade_xml_get_widget (fcdata->gui, "progress_info");
+	fcdata->progress_cancel = glade_xml_get_widget (fcdata->gui, "progress_cancel");
+
+	g_signal_connect (G_OBJECT (fcdata->progress_dialog), 
+			  "delete_event",
+			  G_CALLBACK (folder_copy__interrupt_cb1),
+			  fcdata);
+	g_signal_connect (G_OBJECT (fcdata->progress_cancel), 
+			  "clicked",
+			  G_CALLBACK (folder_copy__interrupt_cb2),
+			  fcdata);
 	
 	gtk_window_set_transient_for (GTK_WINDOW (fcdata->progress_dialog),
 				      GTK_WINDOW (fcdata->window->app));
@@ -2052,6 +2188,8 @@ folder_copy (GThumbWindow   *window,
 
 	/**/
 
+	all_windows_remove_monitor ();
+
 	src_list = g_list_append (src_list, new_uri_from_path (src_path));
 
 	if (fcdata->file_op != FILE_OP_DELETE)
@@ -2061,13 +2199,13 @@ folder_copy (GThumbWindow   *window,
 		char *src_cache;
 		char *dest_cache;
 
-		src_cache = comments_get_comment_dir (src_path, TRUE);
+		src_cache = comments_get_comment_dir (src_path, TRUE, TRUE);
 		if (! path_is_dir (src_cache)) {
 			g_free (src_cache);
-			src_cache = comments_get_comment_dir (src_path, FALSE);
+			src_cache = comments_get_comment_dir (src_path, FALSE, TRUE);
 		}
 
-		dest_cache = comments_get_comment_dir (dest_path, TRUE);
+		dest_cache = comments_get_comment_dir (dest_path, TRUE, TRUE);
 
 		if (path_is_dir (src_cache)) {
 			char *parent_dir = remove_level_from_path (dest_cache);
@@ -2296,7 +2434,7 @@ copy_item__continue1 (GnomeVFSResult result,
 		else
 			error = _("Could not move the folder \"%s\": %s");
 		
-		utf8_name = g_locale_to_utf8 (file_name_from_path (folder), -1, 0, 0, 0);
+		utf8_name = g_filename_to_utf8 (file_name_from_path (folder), -1, 0, 0, 0);
 		message = g_strdup_printf (error, 
 					   utf8_name, 
 					   gnome_vfs_result_to_string (result),
@@ -2308,6 +2446,7 @@ copy_item__continue1 (GnomeVFSResult result,
 						     GTK_DIALOG_MODAL,
 						     GTK_STOCK_DIALOG_ERROR,
 						     message,
+						     NULL,
 						     GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL,
 						     NULL);
 		else 
