@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001 The Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -47,18 +47,21 @@
 #define ICON_NAME_DIRECTORY "gnome-fs-directory"
 
 
-GList               *window_list = NULL;
-FullScreen          *fullscreen;
-char               **file_urls, **dir_urls;
-int                  n_file_urls, n_dir_urls;
-int                  StartInFullscreen;
-int                  StartSlideshow;
-int                  ViewFirstImage = FALSE;
-int                  HideSidebar = FALSE;
-gboolean             ExitAll = FALSE;
+GList          *window_list = NULL;
+FullScreen     *fullscreen;
+char          **file_urls, **dir_urls;
+int             n_file_urls, n_dir_urls;
+int             StartInFullscreen;
+int             StartSlideshow;
+int             ViewFirstImage = FALSE;
+int             HideSidebar = FALSE;
+gboolean        ExitAll = FALSE;
+char           *ImageToDisplay = NULL;
+gboolean        FirstStart = TRUE;
 
 
 static gboolean        view_comline_catalog = FALSE;
+static gboolean        view_single_image = FALSE;
 static GdkPixbuf      *folder_pixbuf = NULL;
 static GThumbWindow   *first_window = NULL;
 static GnomeIconTheme *icon_theme = NULL;
@@ -279,10 +282,8 @@ initialize_data (poptContext pctx)
 
 		if (path_is_dir (path))
 			is_dir = TRUE;
-
 		else if (path_is_file (path))
 			is_dir = FALSE;
-
 		else {
 			g_free (path);
 			continue;
@@ -297,19 +298,24 @@ initialize_data (poptContext pctx)
 			file_urls[n_file_urls++] = path;
 	}
 
-	if (n_file_urls > 0) {
+	if ((n_file_urls == 1) && (file_urls[0] != NULL)) {
+		view_single_image = TRUE;
+
+	} else if (n_file_urls > 1) {
 		/* Create a catalog with the command line list. */
 		Catalog *catalog;
 		char    *catalog_path;
-		char    *catalog_name;
+		char    *catalog_name, *catalog_name_utf8;
 
 		catalog = catalog_new ();
 
-		catalog_name = g_strconcat (_("Command Line"),
-					    CATALOG_EXT,
-					    NULL);
+		catalog_name_utf8 = g_strconcat (_("Command Line"),
+						 CATALOG_EXT,
+						 NULL);
+		catalog_name = g_filename_from_utf8 (catalog_name_utf8, -1, 0, 0, 0);
 		catalog_path = get_catalog_full_path (catalog_name);
 		g_free (catalog_name);
+		g_free (catalog_name_utf8);
 
 		catalog_set_path (catalog, catalog_path);
 		g_free (catalog_path);
@@ -364,7 +370,23 @@ prepare_app ()
 			first_window = current_window;
 	}
 
-	if (view_comline_catalog) {
+	if (view_single_image) {
+		char *image_folder;
+
+		ImageToDisplay = g_strdup (file_urls[0]);
+		HideSidebar = TRUE;
+
+		image_folder = remove_level_from_path (ImageToDisplay);
+		preferences_set_startup_location (image_folder);
+		g_free (image_folder);
+
+		current_window = window_new ();
+		gtk_widget_show (current_window->app);
+
+		if (first_window == NULL)
+			first_window = current_window;
+
+	} else if (view_comline_catalog) {
 		char *catalog_uri;
 		char *catalog_path;
 		char *catalog_name;
@@ -376,12 +398,12 @@ prepare_app ()
 		catalog_uri = g_strconcat ("catalog://", catalog_path, NULL);
 		g_free (catalog_path);
 
-		eel_gconf_set_locale_string (PREF_STARTUP_LOCATION, catalog_uri);
-
+		preferences_set_startup_location (catalog_uri);
 		g_free (catalog_uri);
 
 		ViewFirstImage = TRUE;
 		HideSidebar = TRUE;
+
 		current_window = window_new ();
 		gtk_widget_show (current_window->app);
 
@@ -391,7 +413,7 @@ prepare_app ()
 
 	for (i = 0; i < n_dir_urls; i++) {
 		/* Go to the specified directory. */
-		eel_gconf_set_locale_string (PREF_STARTUP_LOCATION, dir_urls[i]);
+		preferences_set_startup_location (dir_urls[i]);
 
 		current_window = window_new ();
 		gtk_widget_show (current_window->app);
@@ -437,31 +459,6 @@ all_windows_update_bookmark_list ()
 }
 
 
-void 
-all_windows_update_viewer_options ()
-{
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-
-		image_viewer_set_check_type   (IMAGE_VIEWER (window->viewer),
-					       pref_get_check_type ());
-		image_viewer_set_check_size   (IMAGE_VIEWER (window->viewer),
-					       pref_get_check_size ());
-		image_viewer_set_transp_type  (IMAGE_VIEWER (window->viewer),
-					       pref_get_transp_type ());
-		image_viewer_set_zoom_quality (IMAGE_VIEWER (window->viewer),
-					       pref_get_zoom_quality ());
-		image_viewer_set_zoom_change  (IMAGE_VIEWER (window->viewer),
-					       pref_get_zoom_change ());
-		image_viewer_update_view      (IMAGE_VIEWER (window->viewer));
-
-		window_sync_menu_with_preferences (window);
-	}
-}
-
-
 void 
 all_windows_update_browser_options ()
 {
@@ -470,8 +467,8 @@ all_windows_update_browser_options ()
 	for (scan = window_list; scan; scan = scan->next) {
 		GThumbWindow  *window = scan->data;
 		
-		window->file_list->enable_thumbs = eel_gconf_get_boolean (PREF_SHOW_THUMBNAILS);
-		gth_file_list_set_thumbs_size (window->file_list, eel_gconf_get_integer (PREF_THUMBNAIL_SIZE));
+		window->file_list->enable_thumbs = eel_gconf_get_boolean (PREF_SHOW_THUMBNAILS, TRUE);
+		gth_file_list_set_thumbs_size (window->file_list, eel_gconf_get_integer (PREF_THUMBNAIL_SIZE, 95));
 		gth_file_view_set_view_mode (window->file_list->view,
 					     pref_get_view_mode ());
 		window_update_file_list (window);
@@ -505,9 +502,11 @@ all_windows_notify_files_deleted (GList *list)
 	for (scan = window_list; scan; scan = scan->next) {
 		GThumbWindow *window = scan->data;
 
+		/* FIXME
 		if ((window->sidebar_content == GTH_SIDEBAR_DIR_LIST) &&
 		    window->monitor_enabled)
 			continue;
+		*/
 
 		window_notify_files_deleted (window, list);
 	}
@@ -952,7 +951,7 @@ load_session (void)
 		location = gnome_config_get_string (key);
 		g_free (key);
 
-		eel_gconf_set_locale_string (PREF_STARTUP_LOCATION, location);
+		preferences_set_startup_location (location);
 
 		window = window_new ();
 		gtk_widget_show (window->app);
